==> currency_cloud-0.1.0.tar.gz: tool/grind.dart <==
library currency_cloud_dart.tool.grind;

import 'dart:io' hide ProcessException;
import 'dart:mirrors';

import 'package:glob/glob.dart';
import 'package:grinder/grinder.dart';
import 'package:logging/logging.dart';
import 'package:path/path.dart' as path;
import 'package:yaml/yaml.dart';

final Directory projectRoot =
    new Directory.fromUri(currentMirrorSystem().findLibrary(#currency_cloud_dart.tool.grind).uri)
        .parent
        .parent
        .absolute;

final int dartFmtLineLenght = 120;

main(List<String> args) {
  Logger.root.onRecord.listen((record) => log(record.message));
  grind(args);
}

@Task('Runs dartanalyzer and ignores problems listed in the exclude section of analysis_options.yaml')
analyze() async {
  YamlNode yaml = loadYaml(new File(path.join(projectRoot.path, 'analysis_options.yaml')).readAsStringSync());
  Map parsedYaml = yaml.value as Map;
  var excludedDirectories = parsedYaml['analyzer']['exclude'] as List<String>;

  List<Glob> excludedGlobs = excludedDirectories.map((dir) => new Glob(dir));

  var executable = 'dartanalyzer', arguments = [projectRoot.path];
  log("$executable ${arguments.join(' ')}");
  ProcessResult result = Process.runSync(executable, arguments, workingDirectory: projectRoot.path);

  var lines = '${result.stdout}\n${result.stderr}'.split("\n");

  var fileRegExps = <RegExp>[
    new RegExp(r' at (.+):\d+:\d+'),
    new RegExp(r'\((.+), line \d+, col \d+\)'),
    new RegExp(r'(.+) is a part and cannot be analyzed'),
  ];

  var ignoredLinesRegExp = <RegExp>[
    new RegExp(r'^Analyzing /'),
    new RegExp(r'^No issues found!'),
    new RegExp(r'^\d+ warnings and \d+ lints found\.'),
    new RegExp(r'^\d+ hints and \d+ lints found\.'),
    new RegExp(r'^\d+ lints found\.'),
    new RegExp(r'Please pass in a library that contains this part\.'),
  ];

  var ignoredFiles = 0, problems = [], unrecognizedLines = [];
  lines.forEach((line) {
    if (line.trim().isEmpty) return;

    if (!fileRegExps.any((exp) => exp.hasMatch(line))) {
      // This is not a regular file issue
      if (!ignoredLinesRegExp.any((exp) => exp.hasMatch(line))) {
        unrecognizedLines.add(line);
      }
    } else {
      var concerningFileMatch = fileRegExps.firstWhere((exp) => exp.hasMatch(line)).firstMatch(line);
      var filePath = concerningFileMatch.group(1);
      if (excludedGlobs.any((glob) => glob.matches(filePath))) {
        ignoredFiles++;
      } else {
        problems.add(line);
      }
    }
  });

  log('Ignored $ignoredFiles files in $excludedDirectories');

  if (problems.isNotEmpty) {
    log('Those files failed validation:');
    problems.forEach(log);
  }
  if (unrecognizedLines.isNotEmpty) {
    log('Those lines could not be recognized:');
    unrecognizedLines.forEach(log);
  }
  if (problems.isNotEmpty || unrecognizedLines.isNotEmpty) {
    fail('Analyzer discovered problems');
  }
}

@Task()
checkFormat() {
  if (DartFmt.dryRun(projectRoot, lineLength: dartFmtLineLenght))
    fail('Code is not properly formatted. Run `grind format`');
}

@Task()
clean() => defaultClean();

@Task()
format() => DartFmt.format(projectRoot, lineLength: dartFmtLineLenght);

/// returns true if the executable is available
bool isAvailableExecutable(String executable) {
  var runOptions = new RunOptions(environment: Platform.environment, runInShell: true);
  try {
    run('hash', arguments: [executable, '2>/dev/null'], runOptions: runOptions);
    return true;
  } catch (_) {
    return false;
  }
}

@Task()
@Depends(checkFormat, analyze, testUnit)
test() => true;

@Task()
testUnit() => new TestRunner().testAsync(files: new Directory('test'), concurrency: 4);

==> currency_cloud-0.2.0.tar.gz: tool/grind.dart <==
library currency_cloud_dart.tool.grind;

import 'dart:io' hide ProcessException;
import 'dart:mirrors';

import 'package:glob/glob.dart';
import 'package:grinder/grinder.dart';
import 'package:logging/logging.dart';
import 'package:path/path.dart' as path;
import 'package:yaml/yaml.dart';

final Directory projectRoot =
    new Directory.fromUri(currentMirrorSystem().findLibrary(#currency_cloud_dart.tool.grind).uri)
        .parent
        .parent
        .absolute;

final int dartFmtLineLenght = 120;

main(List<String> args) {
  Logger.root.onRecord.listen((record) => log(record.message));
  grind(args);
}

@Task('Runs dartanalyzer and ignores problems listed in the exclude section of analysis_options.yaml')
analyze() async {
  YamlNode yaml = loadYaml(new File(path.join(projectRoot.path, 'analysis_options.yaml')).readAsStringSync());
  Map parsedYaml = yaml.value as Map;
  var excludedDirectories = parsedYaml['analyzer']['exclude'] as List<String>;

  List<Glob> excludedGlobs = excludedDirectories.map((dir) => new Glob(dir));

  var executable = 'dartanalyzer', arguments = [projectRoot.path];
  log('$executable ${arguments.join(' ')}');
  ProcessResult result = Process.runSync(executable, arguments, workingDirectory: projectRoot.path);

  var lines = '${result.stdout}\n${result.stderr}'.split('\n');

  var fileRegExps = <RegExp>[
    new RegExp(r' at (.+):\d+:\d+'),
    new RegExp(r'\((.+), line \d+, col \d+\)'),
    new RegExp(r'(.+) is a part and cannot be analyzed'),
  ];

  var ignoredLinesRegExp = <RegExp>[
    new RegExp(r'^Analyzing /'),
    new RegExp(r'^No issues found!'),
    new RegExp(r'^\d+ warnings and \d+ lints found\.'),
    new RegExp(r'^\d+ hints and \d+ lints found\.'),
    new RegExp(r'^\d+ lints found\.'),
    new RegExp(r'Please pass in a library that contains this part\.'),
  ];

  var ignoredFiles = 0, problems = [], unrecognizedLines = [];
  lines.forEach((line) {
    if (line.trim().isEmpty) return;

    if (!fileRegExps.any((exp) => exp.hasMatch(line))) {
      // This is not a regular file issue
      if (!ignoredLinesRegExp.any((exp) => exp.hasMatch(line))) {
        unrecognizedLines.add(line);
      }
    } else {
      var concerningFileMatch = fileRegExps.firstWhere((exp) => exp.hasMatch(line)).firstMatch(line);
      var filePath = concerningFileMatch.group(1);
      if (excludedGlobs.any((glob) => glob.matches(filePath))) {
        ignoredFiles++;
      } else {
        problems.add(line);
      }
    }
  });

  log('Ignored $ignoredFiles files in $excludedDirectories');

  if (problems.isNotEmpty) {
    log('Those files failed validation:');
    problems.forEach(log);
  }
  if (unrecognizedLines.isNotEmpty) {
    log('Those lines could not be recognized:');
    unrecognizedLines.forEach(log);
  }
  if (problems.isNotEmpty || unrecognizedLines.isNotEmpty) {
    fail('Analyzer discovered problems');
  }
}

@Task()
checkFormat() {
  if (DartFmt.dryRun(projectRoot, lineLength: dartFmtLineLenght))
    fail('Code is not properly formatted. Run `grind format`');
}

@Task()
clean() => defaultClean();

@Task()
format() => DartFmt.format(projectRoot, lineLength: dartFmtLineLenght);

/// returns true if the executable is available
bool isAvailableExecutable(String executable) {
  var runOptions = new RunOptions(environment: Platform.environment, runInShell: true);
  try {
    run('hash', arguments: [executable, '2>/dev/null'], runOptions: runOptions);
    return true;
  } catch (_) {
    return false;
  }
}

@Task()
@Depends(checkFormat, analyze, testUnit)
test() => true;

@Task()
testUnit() => new TestRunner().testAsync(files: new Directory('test'), concurrency: 4);
